@@ -30,5 +30,9 @@ if [ -x /sbin/dmsetup ]; then
     # include dm block var from sysconfig
     mkdir -p etc/sysconfig
     grep DM_BLOCK /etc/sysconfig/kernel > etc/sysconfig/kernel
+	mkdir -p lib/udev/rules.d
+	cp /lib/udev/rules.d/10-dm.rules lib/udev/rules.d
+	cp /lib/udev/rules.d/13-dm-disk.rules lib/udev/rules.d
+	cp /lib/udev/rules.d/95-dm-notify.rules lib/udev/rules.d
     save_var root_dm
 fi
