@@ -1089,7 +1089,7 @@ sub Section2Info {
       				$xen_append =~
 			          s/(.*)${del_console}=\w+\s*(.*)$/$1$2/g;
 			      }
-            $xen_append =~ s/\s*(\S*)\s*$/$1/;
+            $xen_append =~ s/^\s*(\S*)\s*$/$1/;
       			$ret{"xen_append"} = "console=$console $speed $xen_append";
 		    } else {
 		        $ret{"xen_append"} = "console=$console $speed";
@@ -1555,7 +1555,7 @@ sub Info2Section {
       my $xen = $self->UnixPath2GrubPath (delete($sectinfo{"xen"}), $grub_root);
       my $append = (delete($sectinfo{"xen_append"}) || "");
       my $vga = $sectinfo{"vgamode"} || "";
-      $vga = "vgamode=$vga " if $vga ne "";
+      $vga = "vga=mode-$vga " if $vga ne "";
       my $value = "$xen $vga$append";
       push @lines, {
 	"key" => "kernel",
@@ -1861,6 +1861,15 @@ sub Info2Global {
           }
           next;
         }
+  elsif ($key eq "serial"){
+    my $index = 0;
+    foreach my $i (@lines) {
+        last if ($i->{"key"} eq "terminal"); #force order of terminal/serial bnc#650150
+        $index++;
+    }
+    splice @lines, $index, 0, { "key" => $key, "value" => $value };
+    next;
+  }
 	# bool values appear in a config file or not
 	elsif ($type eq "bool") {
 	    next if $value ne "true";
