@@ -15,10 +15,12 @@ BEGIN {
         swig_uuid_set => ["function", "void", "any", "string"],
         swig_sb_ver_get => ["function", "string", "any"],
         swig_sb_ver_set => ["function", "void", "any", "string"],
-        swig_chunk_get => ["function", "integer", "any"],
-        swig_chunk_set => ["function", "void", "any", "integer"],
+        swig_chunkSizeK_get => ["function", "integer", "any"],
+        swig_chunkSizeK_set => ["function", "void", "any", "integer"],
         swig_devices_get => ["function", "string", "any"],
         swig_devices_set => ["function", "void", "any", "string"],
+        swig_spares_get => ["function", "string", "any"],
+        swig_spares_set => ["function", "void", "any", "string"],
     );
 }
 
